/**
 * Copyright (c) 2011-2012 EBM WebSourcing, 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.junit;

import java.util.Set;

import javax.activation.DataHandler;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.xml.transform.Source;

/**
 * @author Marc Jambert - EBM WebSourcing
 */
public interface Message {

    public MessageExchange getMessageExchange();

    public String getFlowStepInterfaceName();

    public String getFlowStepServiceName();

    public String getFlowStepOperationName();

    public String getFlowStepEndpointName();

    public String getFlowStepId();

    public Source getPayload();

    public Source getFault();

    public Source getIn();

    public Source getOut();

    public boolean isInOnly();

    public boolean isInOut();

    public boolean isInOptionalOut();

    public boolean isRobustInOnly();

    /**
     * Retrieve an attachment on the fault
     * 
     * @param attachmentName
     *            The name of the attachment to retrieve
     * @return The attachment matching the given name, or {@code null} is no attachment matching the given name exists.
     */
    public DataHandler getFaultAttachment(final String attachmentName);

    /**
     * Retrieve an attachment on the IN-message
     * 
     * @param attachmentName
     *            The name of the attachment to retrieve
     * @return The attachment matching the given name, or {@code null} is no attachment matching the given name exists.
     */
    public DataHandler getInAttachment(final String attachmentName);

    /**
     * Retrieve an attachment on the OUT-message
     * 
     * @param attachmentName
     *            The name of the attachment to retrieve
     * @return The attachment matching the given name, or {@code null} is no attachment matching the given name exists.
     */
    public DataHandler getOutAttachment(final String attachmentName);

    public boolean addFaultAttachment(final String attachmentName, final DataHandler attachment)
            throws MessagingException;

    public boolean addInAttachment(final String attachmentName, final DataHandler attachment) throws MessagingException;

    public boolean addOutAttachment(final String attachmentName, final DataHandler attachment)
            throws MessagingException;

    /**
     * @return The attachment names available on the fault, or an empty collection, or {@code null} if no fault exists.
     */
    public Set<String> getFaultAttachmentNames();

    /**
     * @return The attachment names available on the request, or an empty collection, or {@code null} if no IN-message
     *         exists.
     */
    public Set<String> getInAttachmentNames();

    /**
     * @return The attachment names available on the reply, or an empty collection, or {@code null} if no OUT-message
     *         exists.
     */
    public Set<String> getOutAttachmentNames();

    public ExchangeStatus getStatus();

    public Exception getError();
}
