/**
 * Copyright (c) 2011-2012 EBM WebSourcing, 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.junit.impl;

import java.io.File;
import java.net.URL;
import java.util.Map.Entry;

import javax.xml.namespace.QName;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.ebmwebsourcing.easycommons.xml.DOMHelper;

/**
 * @author Marc Jambert - EBM WebSourcing
 * @author Victor Noel
 */
public class ComponentConfiguration extends Configuration {

    private final String configurationName;

    public ComponentConfiguration(final String configurationName) {
        this.configurationName = configurationName;
    }

    public String getConfigurationName() {
        return configurationName;
    }

    protected static Element getComponentElement(final Document document) {
        final Element jbi = getJbiElement(document);
        return getElementByTagNameNS(jbi, JBI_NAMESPACE_URI, "component");
    }

    @Override
    Document initialiseJBIDocument(final File installationRoot) {
        final URL defaultComponentJbiUrl = getClass().getClassLoader().getResource("jbi/jbi.xml");
        final Document jbiDocument = (Document) DOMHelper.parseAsDOMSource(defaultComponentJbiUrl).getNode();

        final Element component = getComponentElement(jbiDocument);
        
        // let's set the name
        final Element identification = getElementByTagNameNS(component, JBI_NAMESPACE_URI, "identification");
        addOrReplaceElement(jbiDocument, identification, new QName(JBI_NAMESPACE_URI, "name"),
                this.getConfigurationName());

        // the parameters
        for (final Entry<QName, String> entry : this.getParameters().entrySet()) {
            addOrReplaceElement(jbiDocument, component, entry.getKey(), entry.getValue());
        }

        return jbiDocument;
    }
}
