/**
 * Copyright (c) 2015-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.junit.impl.message;

import java.io.InputStream;
import java.io.StringReader;

import javax.jbi.messaging.Fault;
import javax.jbi.messaging.MessagingException;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;

import org.ow2.petals.component.framework.junit.RequestMessage;
import org.ow2.petals.component.framework.junit.ResponseMessage;
import org.ow2.petals.jbi.xml.BytesSource;

import com.ebmwebsourcing.easycommons.lang.UncheckedException;

/**
 * @author Christophe DENEUX - LInagora
 */
public class FaultToConsumerMessage extends AbstractWrappedToComponentMessage implements ResponseMessage {

    public FaultToConsumerMessage(final RequestMessage request, final byte[] outContent) {
        this(request, outContent == null ? null : new BytesSource(outContent));
    }

    public FaultToConsumerMessage(final RequestMessage request, final String outContent) {
        this(request, outContent == null ? null : new StreamSource(new StringReader(outContent)));
    }

    public FaultToConsumerMessage(final RequestMessage request, final InputStream outContent) {
        this(request, outContent == null ? null : new StreamSource(outContent));
    }

    public FaultToConsumerMessage(final RequestMessage request, final Source outContent) {
        super(request);

        try {
            final Fault fault = this.messageExchange.createFault();
            fault.setContent(outContent);
            this.messageExchange.setFault(fault);
        } catch (final MessagingException e) {
            throw new UncheckedException("Impossible to create a message exchange", e);
        }
    }

    public FaultToConsumerMessage(final RequestMessage request, final FaultToConsumerMessage response) {
        this(request, response.getFault());
    }

    @Override
    public Source getPayload() {
        return getFault();
    }

    @Override
    public boolean isFault() {
        return true;
    }
}
