/**
 * Copyright (c) 2011-2012 EBM WebSourcing, 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.junit.impl.message;

import java.io.InputStream;
import java.io.StringReader;
import java.util.List;

import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;

import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.junit.RequestMessage;
import org.ow2.petals.component.framework.junit.ResponseMessage;
import org.ow2.petals.component.framework.util.jaxb.JBIAttachmentMarshaller;
import org.ow2.petals.jbi.xml.BytesSource;

import com.ebmwebsourcing.easycommons.lang.UncheckedException;
import com.ebmwebsourcing.easycommons.stream.EasyByteArrayOutputStream;

/**
 * @author Marc Jambert - EBM WebSourcing
 */
public class ResponseToConsumerMessage extends AbstractWrappedToComponentMessage implements ResponseMessage {

    public ResponseToConsumerMessage(final RequestMessage request, final Object outContent,
            final Marshaller marshaller) {
        this(request);

        final EasyByteArrayOutputStream ebaos = new EasyByteArrayOutputStream();
        try {
            final NormalizedMessage outNm = this.messageExchange.createMessage();
            marshaller.setProperty(Marshaller.JAXB_ENCODING, "UTF-8");
            marshaller.setAttachmentMarshaller(new JBIAttachmentMarshaller(outNm));
            marshaller.marshal(outContent, ebaos);
            outNm.setContent(new BytesSource(ebaos.toRawByteArray()));
            this.messageExchange.setMessage(outNm, Exchange.OUT_MESSAGE_NAME);
        } catch (final MessagingException | JAXBException e) {
            throw new UncheckedException("Impossible to create a message exchange", e);
        }
    }

    public ResponseToConsumerMessage(final RequestMessage request, final byte[] outContent) {
        this(request, outContent, (List<Attachment>) null);
    }

    public ResponseToConsumerMessage(final RequestMessage request, final byte[] outContent,
            final List<Attachment> attachments) {
        this(request, outContent == null ? null : new BytesSource(outContent), attachments);
    }

    public ResponseToConsumerMessage(final RequestMessage request, final String outContent) {
        this(request, outContent, (List<Attachment>) null);
    }

    public ResponseToConsumerMessage(final RequestMessage request, final String outContent,
            final List<Attachment> attachments) {
        this(request, outContent == null ? null : new StreamSource(new StringReader(outContent)), attachments);
    }

    public ResponseToConsumerMessage(final RequestMessage request, final InputStream outContent) {
        this(request, outContent, (List<Attachment>) null);
    }

    public ResponseToConsumerMessage(final RequestMessage request, final InputStream outContent,
            final List<Attachment> attachments) {
        this(request, outContent == null ? null : new StreamSource(outContent), attachments);
    }


    public ResponseToConsumerMessage(final RequestMessage request, final Source outContent) {
        this(request, outContent, (List<Attachment>) null);
    }

    public ResponseToConsumerMessage(final RequestMessage request, final Source outContent,
            final List<Attachment> attachments) {
        this(request);

        try {
            final NormalizedMessage outNm = this.messageExchange.createMessage();
            outNm.setContent(outContent);
            this.messageExchange.setMessage(outNm, Exchange.OUT_MESSAGE_NAME);
            if (attachments != null) {
                for (final Attachment attachment : attachments) {
                    outNm.addAttachment(attachment.getId(), attachment.getContent());
                }
            }
        } catch (final MessagingException e) {
            throw new UncheckedException("Impossible to create a message exchange", e);
        }
    }

    public ResponseToConsumerMessage(final RequestMessage request) {
        super(request);
    }

    public ResponseToConsumerMessage(final RequestMessage request, final ResponseMessage response) {
        this(request, response.getOut());
    }

    @Override
    public Source getPayload() {
        return getOut();
    }

    @Override
    public boolean isFault() {
        return false;
    }
}
