/**
 * Copyright (c) 2011-2012 EBM WebSourcing, 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.junit.impl.message;

import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessagingException;
import javax.xml.transform.Source;

import org.junit.Assert;
import org.ow2.petals.component.framework.junit.RequestMessage;
import org.ow2.petals.component.framework.junit.StatusMessage;

import com.ebmwebsourcing.easycommons.lang.UncheckedException;

/**
 * Status actually means DONE! (but errors are possible)
 * 
 * This one is for InOnly and RobustInOnly mainly
 * 
 * @author Victor Noel
 */
public class StatusToConsumerMessage extends AbstractWrappedToComponentMessage implements StatusMessage {

    public StatusToConsumerMessage(final RequestMessage request, final ExchangeStatus status) {
        super(request);

        try {
            this.messageExchange.setStatus(status);
        } catch (final MessagingException e) {
            throw new UncheckedException("Impossible to create a message exchange", e);
        }
    }

    public StatusToConsumerMessage(final RequestMessage request, final Exception error) {
        super(request);

        this.messageExchange.setError(error);
    }

    public StatusToConsumerMessage(final RequestMessage request, final StatusMessage status) {
        this(request, status.getStatus());
    }

    @Override
    public Source getPayload() {
        Assert.fail("This is not meant to be called on a StatusMessage");
        return null;
    }
}
