/**
 * Copyright (c) 2016-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.junit.impl.mock;

import java.net.URI;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.MessagingException;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import org.mockito.Mockito;
import org.ow2.petals.component.framework.junit.TestMessageExchangeFactory;
import org.ow2.petals.jbi.messaging.exchange.AbstractMessageExchangeFactory;
import org.ow2.petals.jbi.messaging.exchange.MessageExchangeWrapper;
import org.ow2.petals.jbi.messaging.exchange.PetalsMessageExchange;
import org.ow2.petals.jbi.messaging.exchange.impl.MessageExchangeImpl;
import org.ow2.petals.jbi.servicedesc.endpoint.Location;
import org.w3c.dom.Document;

import com.ebmwebsourcing.easycommons.lang.UncheckedException;
import com.ebmwebsourcing.easycommons.uuid.QualifiedUUIDGenerator;

/**
 * This is meant to be used by tests to create exchanges easily
 * 
 * @author vnoel
 *
 */
public class TestMessageExchangeFactoryImpl extends AbstractMessageExchangeFactory
        implements TestMessageExchangeFactory {

    protected static final QualifiedUUIDGenerator GENERATOR = new QualifiedUUIDGenerator("tests");

    private final MockEndpointDirectory directory;

    public TestMessageExchangeFactoryImpl(final MockEndpointDirectory directory, final Logger log) {
        super(null, null, null, new Location("MockContainer", "unit-test"), createFakeComponentContext(), log);
        this.directory = directory;
    }

    @Override
    protected PetalsMessageExchange buildExchange(final URI pattern, final Location consumerLocation)
            throws MessagingException {
        return new MessageExchangeImpl(GENERATOR.getNewID(), pattern, consumerLocation);
    }

    private static ComponentContext createFakeComponentContext() {
        final ComponentContext mock = Mockito.mock(ComponentContextStub.class);
        Mockito.when(mock.getEndpointsForService(Mockito.any(QName.class))).thenCallRealMethod();
        try {
            Mockito.when(mock.getEndpointDescriptor(Mockito.any(ServiceEndpoint.class))).thenCallRealMethod();
        } catch (final JBIException e) {
            // TODO not sure how to handle that with Mockito... this should not happen anyway!
            throw new UncheckedException(e);
        }
        return mock;
    }

    /**
     * Add other methods used by {@link AbstractMessageExchangeFactory} if needed!
     */
    public abstract class ComponentContextStub implements ComponentContext {

        @Override
        public ServiceEndpoint[] getEndpointsForService(final QName serviceName) {
            return directory.resolveEndpointsForService(serviceName).toArray(new ServiceEndpoint[0]);
        }

        @Override
        public Document getEndpointDescriptor(final ServiceEndpoint endpoint) throws JBIException {
            final MockServiceEndpoint mock = directory.resolveEndpoint(endpoint.getServiceName(),
                    endpoint.getEndpointName());
            return mock != null ? mock.getDescription() : null;
        }
    }

    @Override
    public PetalsMessageExchange createExchange(final URI pattern) throws MessagingException {
        return buildExchange(pattern);
    }

    @Override
    public MessageExchangeWrapper createWrappedExchange(URI pattern) throws MessagingException {
        return buildExchangeDecorator(pattern);
    }

}
